//
//  main.c
//
//  Program main / Thermo Hygrometer SDcard logger 0.2
//  Copyright (C) 2009-2010 Toyohiko Togashi tog001@nifty.com
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		Thermometer and Hygrometer with memory record.
//		Long time record to SDcard media.
//
//	Spec
//		Display	device	: 2x16 Character LCD module
//		Display item	: "yyyy-mm-dd HH:MM:SS", C, %RH, Battery gage, etc...
//		Memory card		: SDcard, under 2GB, FAT16 file system format
//		Record format	: CSV (Camma separated value)
//		File name		: "TH.CSV" fixed, every append.
//		Record item		: "yyyy-mm-dd HH:MM:SS", C, %RH
//		Sample interval	: Select 1sec, 10sec, 60sec, 10min
//
//	Update history
//	---------- ----- -------------------------------------------
//	2009.07.03 v0.0  First cording
//	2009.07.06 v0.0a Alpha
//	2009.07.08 v0.0a Append battery status by SMP & link error(auto->static)
//	2009.07.17 v0.0a Append battery shutdown
//	2009.07.18 v0.0b Supported PCB R4, Thermistor=103AT-2 R1=33K
//	2009.07.19 v0.0b bug fix, minus tempture output
//	2009.07.25 v0.1  First release
//	2009.07.28 v0.1  Table update to thermistor (90%VR)
//	2009.08.04 v0.2  HI-TECH C -> ImageCraft C Compiler
//	2009.08.05 v0.2  HP15P load stabilize, Ri=33K->1K(R1LO option) -> not used
//	2009.12.18 v0.2a bug fix, under 0.8"C is invalid
//	2009.12.18 v0.2a Modify battery gauge level
//	2009.12.24 v0.2b Cypress MMC/SDcard user module -> Private SDcard support
//	2010.01.18 v0.2c update SDcard support 0.1a 
//

#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules

#include <limits.h>
//#include <math.h>
//#include <stdio.h>					// DEL 2009.08.04
#include <string.h>						// ADD 2009.08.04
#include <stdlib.h>						// ADD 2009.07.19
#include "i2c_lcd.h"
#include "tsprintf5.h"					// ADD 2009.08.04
#include "sd.h"							// ADD 2009.12.24
#include "PSoCGPIOINT.h"


//
// Config
//
#define PCBR4							// ADD 2009.07.18
//#define R1LO							// ADD 2009.08.05  Special PCB, R1 is low value < 1K
#define SAMPLE_COUNT 100				// ADC 1 measure unit


#ifdef PCBR4							// ADD 2009.07.18
#ifdef R1LO								// ADD 2009.08.05
#define R1VALUE  970					// ADD 2009.08.05 [ohm] Humidity senser series registor 1K
#define ISHI103AT						// ADD 2009.08.05 Thermistor=Ishizuka 103AT-2
#define HOFFSET  -40					// ADD 2009.08.05 Absolute 0.1%RH offset
#else	//R1LO							// ADD 2009.08.05
#define R1VALUE  33000					// ADD 2009.07.18 [ohm] Humidity senser series registor 33K
#define ISHI103AT						// ADD 2009.07.18 Thermistor=Ishizuka 103AT-2
#define HOFFSET  -80					// ADD 2009.08.05 Absolute 0.1%RH offset
#endif	//R1LO							// ADD 2009.08.05
#else	//PCBR4							// ADD 2009.07.18
#define R1VALUE  30000					// CHG 2009.07.18 [ohm] Humidity senser series registor
#define NTSA0XV103FE1B0					// ADD 2009.07.18 Thermistor=Murata
#endif	//PCBR4							// ADD 2009.07.18

#define MAX(a,b) ((a)>(b)?(a):(b))
#define MIN(a,b) ((a)<(b)?(a):(b))


//
// Global area
//
//#define LOGBUFSZ 10					// DEL 2009.12.24
//static char logbuf_yy[LOGBUFSZ];		// DEL 2009.12.24
//static char logbuf_mm[LOGBUFSZ];		// DEL 2009.12.24
//static char logbuf_dd[LOGBUFSZ];		// DEL 2009.12.24
//static char logbuf_hh[LOGBUFSZ];		// DEL 2009.12.24
//static char logbuf_mi[LOGBUFSZ];		// DEL 2009.12.24
//static char logbuf_ss[LOGBUFSZ];		// DEL 2009.12.24
//static int  logbuf_t[LOGBUFSZ];		// DEL 2009.12.24
//static int  logbuf_h[LOGBUFSZ];		// DEL 2009.12.24
//static char logbufnum = 0;			// DEL 2009.12.24
static int	iv = 1;						// Log interval [sec]
//static uchar fp = SDCard_1_MAXFILES;			// DEL 2009.12.24  SDcard file pointer

//
// Table refrence & Linear interpolation conversion function
//
struct litbl {
	int f;								// return value
	long x;								// reference value
};

#ifdef NTSA0XV103FE1B0					// ADD 2009.07.18
// DELETE NEXT 13LINES
//const struct litbl litemp[] = {
//	 800,	 501,						// 10xC , A/D count
//	 700,	 453,
//	 600,	 388,
//	 500,	 303,
//	 400,	 196,
//	 300,	  69,
//	 250,	   0,
//	 200,	 -72,
//	 100,	-215,
//	  00,	-345,
//	-100,	-451};
//#define LITEMPSZ 11
const struct litbl litemp[] = {			// ADD 2009.07.28
//	[C]*10	A/Dcount					// ADD 2009.07.28
	 800,	 386,						// ADD 2009.07.28
	 700,	 342,						// ADD 2009.07.28
	 600,	 284,						// ADD 2009.07.28
	 500,	 208,						// ADD 2009.07.28
	 400,	 112,						// ADD 2009.07.28
	 300,	  -3,						// ADD 2009.07.28
	 200,	-130,						// ADD 2009.07.28
	 100,	-258,						// ADD 2009.07.28
	  00,	-376,						// ADD 2009.07.28
	-100,	-471,						// ADD 2009.07.28
	-200,	-542};						// ADD 2009.07.28
#define LITEMPSZ 11						// ADD 2009.07.28
#endif									// ADD 2009.07.18

#ifdef ISHI103AT						// ADD 2009.07.18
// DELETE NEXT 15LINES
//const struct litbl litemp[] = {		// ADD 2009.07.18
//	 900,	 503,						// ADD 2009.07.18
//	 800,	 464,						// ADD 2009.07.18
//	 700,	 413,						// ADD 2009.07.18
//	 600,	 348,						// ADD 2009.07.18
//	 500,	 268,						// ADD 2009.07.18
//	 400,	 171,						// ADD 2009.07.18
//	 300,	  59,						// ADD 2009.07.18
//	 250,	   0,						// ADD 2009.07.18
//	 200,	 -62,						// ADD 2009.07.18
//	 100,	-186,						// ADD 2009.07.18
//	  00,	-302,						// ADD 2009.07.18
//	-100,	-403,						// ADD 2009.07.18
//	-200,	-483};						// ADD 2009.07.18
//#define LITEMPSZ 13					// ADD 2009.07.18
const struct litbl litemp[] = {			// ADD 2009.07.18
//	[C]*10	A/Dcount					// ADD 2009.07.28
	 800,	 352,						// ADD 2009.07.28
	 700,	 306,						// ADD 2009.07.28
	 600,	 248,						// ADD 2009.07.28
	 500,	 176,						// ADD 2009.07.28
	 400,	  89,						// ADD 2009.07.28
	 300,	 -12,						// ADD 2009.07.28
	 200,	-121,						// ADD 2009.07.28
	 100,	-232,						// ADD 2009.07.28
	  00,	-337,						// ADD 2009.07.28
	-100,	-427,						// ADD 2009.07.28
	-200,	-500};						// ADD 2009.07.28
#define LITEMPSZ 11						// ADD 2009.07.28
#endif									// ADD 2009.07.18

const struct litbl lihumi[] = {
	 200,	8000000,					// 10x%RH , Ohm
	 300,	1000000,
	 400,	 200000,
	 500,	  60000,
	 600,	  22000,
	 700,	   8300,
	 800,	   3400,
	 900,	   1600,
	1000,	    800};
#define LIHUMISZ 9

//int lifunc(long x, struct litbl tbl[], char tblsz) {		// DEL 2009.08.04
int lifunc(long x, const struct litbl tbl[], char tblsz) {	// ADD 2009.08.04
	int f;
	char i;

	// Table search
	f = INT_MIN;
	for(i = 0; i < tblsz; i++) {
		if (x > tbl[i].x) {
			if (i == 0) {
				f = INT_MAX;
			// Linear interpolation
			} else {
				long a;
				long b;
				int c;
				int d;

				a = tbl[i-1].x;
				b = tbl[i].x;
				c = tbl[i-1].f;
				d = tbl[i].f;
				f = (x - b) * (c - d) / (a - b) +  d;
			}
			break;
		}
	}
	return(f);
//};									// DEL 2009.08.04
}										// ADD 2009.08.04

//
//	Interrupt Service Routine
//
//#pragma interrupt_handler Timer16_1_ISR					// DEL 2009.12.24
#pragma interrupt_handler PSoC_GPIO_ISR Timer16_1_ISR		// ADD 2009.12.24
// PSoCGPIOINT.asm, Insert "ljmp	_PSoC_GPIO_ISR"			// ADD 2009.12.24

// SDcard eject ISR											// ADD 2009.12.24
void PSoC_GPIO_ISR(void) {									// ADD 2009.12.24
#ifdef AAA
	// SD_CD pin											// ADD 2009.12.24
	if (SD_CD_Data_ADDR & SD_CD_MASK) {						// ADD 2009.12.24
		if (fd >= 0) {										// ADD 2009.12.24
			if (sd_close(fd) < 0) {							// ADD 2009.12.24
				I2C_LCD_Position(1,0);						// ADD 2009.12.24
				I2C_LCD_PrCString("Close error     ");		// ADD 2009.12.24
			}												// ADD 2009.12.24
			if (sd_unmount() < 0) {							// ADD 2009.12.24
				I2C_LCD_Position(1,0);						// ADD 2009.12.24
				I2C_LCD_PrCString("Unmount error   ");		// ADD 2009.12.24
			}												// ADD 2009.12.24
			fd = -1;										// ADD 2009.12.24
			I2C_LCD_Icon(6, 0x00);							// ADD 2009.12.24
		}													// ADD 2009.12.24
	}														// ADD 2009.12.24
#endif
	return;													// ADD 2009.12.24
}															// ADD 2009.12.24

// 1 sec interval timer ISR
static char ddom[] = {0, 31,28,31,30,31,30,31,31,30,31,30,31};
static char yy =  10;
static char mm =  1;
static char dd =  1;
static char hh =  0;
static char mi =  0;
static char ss =  0;
// void interrupt Timer16_1_ISR(void) @ 0x40 {	// DBB20 digital block  DEL 2009.08.04
void Timer16_1_ISR(void) {				// ADD 2009.08.04
	ss++;
	if (ss > 59) {
		ss = 0;
		mi++;
		if (mi > 59) {
			mi = 0;
			hh++;
			if (hh > 23) {
				hh = 0;
				dd++;
				if (((yy % 4) == 0) && ((yy % 100) != 0)) {
					ddom[2] = 29;
				} else {
					ddom[2] = 28;
				}
				if (dd > ddom[mm]) {
					dd = 1;
					mm++;
					if (mm > 12) {
						mm = 1;
						yy++;
					}
				}
			}
		}
	}
	return;
}

//
// Date Time key input & modify
//
//void dateset() {						// DEL 2009.08.04
void dateset(void) {					// ADD 2009.08.04
	char y,m,d,h,n;
	char p;

	y = yy;
	m = mm;
	d = dd;
	h = hh;
	n = mi;
	p = 1;
	I2C_LCD_Icon(15, 0x10);				// Clock mark ON
	I2C_LCD_Control(I2C_LCD_CURSOR_BLINK);

	for(;;) {
		char buf[18];					// ADD 2009.08.04
		int y2;							// ADD 2009.08.04

		// display
		y2 = y + 2000;											// ADD 2009.08.04
		I2C_LCD_Position(0, 0);
//		printf("%4d.%02d.%02d %02d:%02d", y+2000, m, d, h, n);	// DEL 2009.08.04
		tsprintf5(buf,											// ADD 2009.08.04
			"%4d.%02hhd.%02hhd %02hhd:%02hhd",					// ADD 2009.08.04
			&y2, &m, &d, &h, &n);								// ADD 2009.08.04
		I2C_LCD_PrString((CHAR *)buf);							// ADD 2009.08.04
		switch(p) {
		case 1:
			I2C_LCD_Position(0, 3);
			break;
		case 2:
			I2C_LCD_Position(0, 6);
			break;
		case 3:
			I2C_LCD_Position(0, 9);
			break;
		case 4:
			I2C_LCD_Position(0, 12);
			break;
		default:
			I2C_LCD_Position(0, 15);
			break;
		}
		
		// Key off wait
		while (!(Mode_SW_Data_ADDR & Mode_SW_MASK));
		while (!(Incr_SW_Data_ADDR & Incr_SW_MASK));

		// Key push wait
		while ((Mode_SW_Data_ADDR & Mode_SW_MASK) && (Incr_SW_Data_ADDR & Incr_SW_MASK));

		// Key push
		if (!(Mode_SW_Data_ADDR & Mode_SW_MASK)) {
			p++;
			if (p > 5) break;			// forever exit
		}
		if (!(Incr_SW_Data_ADDR & Incr_SW_MASK)) {
			switch(p) {
			case 1:
				y++;
				if (y > 99) {			// ADD 2009.08.04
					y = 9;				// ADD 2009.08.04
				}						// ADD 2009.08.04
				break;
			case 2:
				m++;
				if (m > 12) {
					m = 1;
				}
				break;
			case 3:
				d++;
				if (d > ddom[m]) {
					d = 1;
				}
				break;
			case 4:
				h++;
				if (h > 23) {
					h = 0;
				}
				break;
			default:
				n++;
				if (n > 59) {
					n = 0;
				}
				break;
			}
		}

		// Chattering cancel
		I2C_LCD_Delay1mTimes(200);		// CHG 2009.07.23  100ms->200ms
	}
	
	// Fix
	yy = y;
	mm = m;
	dd = d;
	hh = h;
	mi = n;
	ss = 0;
	I2C_LCD_Control(I2C_LCD_DISP_ON);
	I2C_LCD_Icon(15, 0);
	
	return;
}

//
// Battery status
//
// ADD 2009.07.08
//char getbatt() {						// DEL 2009.08.04
char getbatt(void) {					// ADD 2009.08.04
	int b;
	int a;
	int d;
	char s;

	// Get SMP gate open count
	b = Counter16_1_wReadCounter();
	Counter16_1_Start();
	I2C_LCD_Delay1mTimes(1);
	Counter16_1_Stop();
	a = Counter16_1_wReadCounter();
	d = b - a;
//	I2C_LCD_Position(1,0);
//	printf("Pwr=%d ", d);
//	if (d < 50) {						// DEL 2009.12.18
	if (d < 40) {						// ADD 2009.12.18
		s = 0;							// Full
//	} else if (d < 100) {				// DEL 2009.12.18
	} else if (d < 60) {				// ADD 2009.12.18
		s = 1;							// Mid
//	} else if (d < 150) {				// DEL 2009.12.18
	} else if (d < 100) {				// ADD 2009.12.18
		s = 2;							// Low
//	} else if (d < 225) {				// CHG 2009.07.17
	} else if (d < 150) {				// ADD 2009.12.18
		s = 3;							// Empty
	} else {							// ADD 2009.07.17
		s = 4;							// CHG 2009.07.17  Critical level
	}
	return(s);
}
const char bs2icon[] = {0x1e, 0x1a, 0x12, 0x02, 0x02};

//
// *** M a i n ***
//
void main(void)							// UPD 2009.12.24
{
	char sw;							// Push switch check counter
	char bs;							// ADD 2009.07.08 Battery status
	char bbs;							// ADD 2009.07.08 Before "bs"
	signed char	fd;						// ADD 2009.12.24

	//
	// Congigure
	//
	Mode_SW_DataShadow |= Mode_SW_MASK;	// Port input mode set
	Mode_SW_Data_ADDR   = Mode_SW_DataShadow;  
	Incr_SW_DataShadow |= Incr_SW_MASK;
	Incr_SW_Data_ADDR   = Incr_SW_DataShadow;  
	SD_CD_DataShadow   |= SD_CD_MASK;		// SDCard module's CD signal pin is pull up 5.4[Kohm] -> bias current=611uA
	SD_CD_Data_ADDR     = SD_CD_DataShadow;	// External pull up registor & myself control

	//
	// init
	//
	bs = getbatt();						// ADD 2009.07.17  Battery check
	if (bs >= 4) {						// ADD 2009.07.17  No power ?
//		VLT_CR |= VLT_CR_SMP;			// ADD 2009.07.17  SMP off
		ABF_CR0 = 0x00;					// ADD 2009.07.17  Analog buffer off
		ARF_CR  = 0x00;					// ADD 2009.07.17  Analog ref off
		for(;;) {						// ADD 2009.07.17
//			M8C_Stop;					// ADD 2009.12.18
			M8C_Sleep;					// ADD 2009.07.17
		}								// ADD 2009.07.17
	}									// ADD 2009.07.17
	M8C_EnableGInt;
	M8C_EnableIntMask(INT_MSK0, INT_MSK0_GPIO);	// ADD 2009.12.24
	Timer16_1_Start();					// 1sec interval timer
	Timer16_1_EnableInt();
//	SDCard_1_Start();					// DEL 2009.12.24
	Bias_Power_Start();
	LCD_Power_Start();
	I2C_LCD_Start();
	sw  = 0;
	bbs = 99;							// ADD 2009.07.08
	fd = -1;							// ADD 2009.12.24

	// LCD Hello message
	LCD_Power_Switch(1);
	I2C_LCD_Init();
//	printf("ThermoHygroMeter");				// DEL 2009.08.04
	I2C_LCD_PrCString("ThermoHygroMeter");	// ADD 2009.08.04
	I2C_LCD_Position(1,0);
//	printf(" V0.1  2009.7.25");				// DEL 2009.08.04
//	I2C_LCD_PrCString(" V0.2  2009.8.4 ");	// ADD 2009.08.04
//	I2C_LCD_PrCString(" V0.2  2009.8.5 ");	// ADD 2009.08.05
//	I2C_LCD_PrCString("V0.2a 2009.12.18");	// ADD 2009.12.18
//	I2C_LCD_PrCString("V0.2b 2009.12.24");	// ADD 2009.12.24
	I2C_LCD_PrCString("V0.2c 2010.01.18");	// ADD 2010.01.18
	{
		int i;
		
		for(i = 0; i < 10; i++) {
			I2C_LCD_Delay1mTimes(200);
		}
	}
	I2C_LCD_Icon(7, 0x00);				// Up/Down mark OFF

	//
	// 1 log/display loop
	//
	for(;;){
		int vt;							// Tempture sensor [A/D count]
		int vspp;						// Humidity sensor + Ri	: Vsource p-p [A/D count]
		int vipp;						// Ri(series register)  : Vi p-p [A/D count]
//		float rs;						// Humidity sensor Z [ohm]
		long rs;						// Humidity sensor Z [ohm]
		int h;							// [%RH]*10
		int t;							// ["C]*10
		char buf[20];					// ADD 2009.12.24
		signed char t1;					// ADD 2009.12.24
		signed char t2;					// ADD 2009.12.24
		signed char h1;					// ADD 2009.12.24
		signed char h2;					// ADD 2009.12.24
		int y2;							// ADD 2009.12.24

		//
		// Measure ADC
		//
		{
			int i;
			int v1max, v1min;
			int v2max, v2min;
			int v3sum;

			// PSoC block startup
			Bias_Power_Switch(1);

			PWM8_1_Start();					// 1KHz source
    		Counter8_1_Start();				// BPF clock
			PGA_0_Start(PGA_0_LOWPOWER);	// PWM8->BPF bridge
    		BPF2_1_Start(BPF2_1_LOWPOWER);	// Square wave -> Sin wave
			I2C_LCD_Delay1mTimes(30);		// Stable wait

			PGA_1_Start(PGA_1_MEDPOWER);	// Humidity sensor Vi
    		PGA_2_Start(PGA_2_MEDPOWER);	// Humidity sensor Vsource
    		PGA_3_Start(PGA_3_MEDPOWER);	// Tempture sensor
#ifdef R1LO									// ADD 2009.08.05
			if (h < 500) {					// ADD 2009.08.05  50.0%RH
				PGA_1_SetGain(PGA_1_G24_0);	// ADD 2009.08.05
			} else {						// ADD 2009.08.05
				PGA_1_SetGain(PGA_1_G1_00);	// ADD 2009.08.05
			}								// ADD 2009.08.05
#endif										// ADD 2009.08.05
			TRIADC_1_Start(TRIADC_1_MEDPOWER);

			// ADC sampling
			v1max = v2max = INT_MIN;
			v1min = v2min = INT_MAX;
			v3sum = 0;
			for(i = 0; i < SAMPLE_COUNT; i++){
				int v1, v2, v3;

				TRIADC_1_GetSamples(1);
				while(TRIADC_1_fIsDataAvailable() == 0);
				v1 = TRIADC_1_iGetData1();
				v2 = TRIADC_1_iGetData2();
				v3 = TRIADC_1_iGetData3ClearFlag();
				v1max = MAX(v1max, v1);
				v1min = MIN(v1min, v1);
				v2max = MAX(v2max, v2);
				v2min = MIN(v2min, v2);
//				v3sum += v3;				// DEL 2009.12.18
				if (i & 1) {				// ADD 2009.12.18
					v3sum += v3;			// ADD 2009.12.18
				}							// ADD 2009.12.18
			}
			vipp = v1max - v1min;
			vspp = v2max - v2min;
//			vt   = v3sum / SAMPLE_COUNT;	// DEL 2009.12.18
			vt   = v3sum / (SAMPLE_COUNT/2);// ADD 2009.12.18
			
#ifdef R1LO									// ADD 2009.08.05
			if (h < 500) {					// ADD 2009.08.05
				vspp = vspp * 24;			// ADD 2009.08.05
			}								// ADD 2009.08.05
#endif										// ADD 2009.08.05
			
			// Power Consumption save
			TRIADC_1_Stop();
    		PGA_3_Stop();
    		PGA_2_Stop();
    		PGA_1_Stop();
   			BPF2_1_Stop();
    		PGA_0_Stop();
			Counter8_1_Stop();
    		PWM8_1_Stop();
			Bias_Power_Switch(0);
		}

		//
		// Calcurate tempture
		//
		t = lifunc(vt, litemp, LITEMPSZ);
		t = MAX(t, -999);				// ADD 2009.07.19
		t = MIN(t,  999);				// ADD 2009.07.19

		//
		// Calcutate relative humidity
		//
		if (vipp != 0) {
//			rs = (float)R1VALUE * (float)vspp / (float)vipp - (float)R1VALUE;	// more memory
//			h = 338.2 * pow(rs, -0.1765);										// more memory
			rs = (long)R1VALUE * (long)vspp / vipp - R1VALUE;
			h = lifunc(rs, lihumi, LIHUMISZ);	// %RH
//			h += ((250 - t) * 67 / 100);		// Temp adjust 1.0"C:-0.67%		DEL 2009.08.05
			h += ((250 - t) * 8 / 10);			// Temp adjust +1["C]:-0.8[%RH]	ADD 2009.08.05
#ifdef HOFFSET									// ADD 2009.08.05
			h += HOFFSET;						// ADD 2009.08.05  Constant offset
#endif											// ADD 2009.08.05
			h = MAX(h,   0);					// ADD 2009.07.19
			h = MIN(h, 999);					// ADD 2009.07.19
		}

		//
		// Display refresh
		//
		t1 = t / 10;							// ADD 2009.12.24
		t2 = abs(t) % 10;						// ADD 2009.12.24
		h1 = h / 10;							// ADD 2009.12.24
		h2 = h % 10;							// ADD 2009.12.24
		y2 = yy + 2000;							// ADD 2009.12.24

// DELETE NEXT 5 LINES  2009.08.04
//		I2C_LCD_Position(0,0);
//		printf("%3d.%d\362C %2d.%d%%RH ",		// UPD 2009.07.19
//				t/10, abs(t)%10, h/10, h%10);	// UPD 2009.07.19
//		I2C_LCD_Position(1,0);
//		printf("%4d.%2d.%2d %2d:%02d", yy+2000, mm, dd, hh, mi);
		{										// ADD 2009.08.04
#ifdef DELETE_START								// ADD 2009.12.24
			char buf[18];						// ADD 2009.08.04
			signed char t1;						// ADD 2009.08.04
			signed char t2;						// ADD 2009.08.04
			signed char h1;						// ADD 2009.08.04
			signed char h2;						// ADD 2009.08.04
			int y2;								// ADD 2009.08.04
												// ADD 2009.08.04
			t1 = t / 10;						// ADD 2009.08.04
			t2 = abs(t) % 10;					// ADD 2009.08.04
			h1 = h / 10;						// ADD 2009.08.04
			h2 = h % 10;						// ADD 2009.08.04
			y2 = yy + 2000;						// ADD 2009.08.04
#endif // DELETE_END							// ADD 2009.12.24
												// ADD 2009.08.04
			tsprintf5(buf,						// ADD 2009.08.04
				"%3hhd.%hhd\362C %2hhd.%hhd%%RH ",	// ADD 2009.08.04
				&t1, &t2, &h1, &h2, 0);			// ADD 2009.08.04
			if ((t1 == 0) && (t < 0)) {			// ADD 2009.12.18  Special... -0.1 ~ -0.9
				buf[1] = '-';					// ADD 2009.12.18
			}									// ADD 2009.12.18
//			tsprintf5(buf,						// ADD 2009.08.05
//				"%hhd.%hhd %d %d ",				// ADD 2009.08.05
//				&h1, &h2, &vspp, &vipp, 0);		// ADD 2009.08.05
			I2C_LCD_Position(0,0);				// ADD 2009.08.04
			I2C_LCD_PrString((CHAR *)buf);		// ADD 2009.08.04
												// ADD 2009.08.04
			tsprintf5(buf,						// ADD 2009.08.04
				"%4d.%2hhd.%2hhd %2hhd:%02hhd",	// ADD 2009.08.04
				&y2, &mm, &dd, &hh, &mi);		// ADD 2009.08.04
			I2C_LCD_Position(1,0);				// ADD 2009.08.04
			I2C_LCD_PrString((CHAR *)buf);		// ADD 2009.08.04
		}										// ADD 2009.08.04

		//
		// SDcard write
		//
		if (fd >= 0) {									// ADD 2009.12.24
			static char bef_mi = 0;						// ADD 2009.12.24
														// ADD 2009.12.24
			I2C_LCD_Icon(9, 0x10);						// ADD 2009.12.24  Lock icon on
			tsprintf5(buf,"\"%4d-%02hhd-%02hhd %02hhd:%02hhd:",&y2,&mm,&dd,&hh,&mi);	// ADD 2009.12.24
			if (sd_write(fd, buf, 18) != 18) {			// ADD 2009.12.24
				I2C_LCD_Position(0,0);					// ADD 2009.12.24
				I2C_LCD_PrCString(" <Write error>  ");	// ADD 2009.12.24
			}											// ADD 2009.12.24
			tsprintf5(buf,"%02hhd\",%3hhd.%hhd,%2hhd.%hhd\r\n",&ss,&t1,&t2,&h1,&h2);	// ADD 2009.12.24
			if ((t1 == 0) && (t < 0)) {					// ADD 2009.12.24  Special... -0.1 ~ -0.9
				buf[5] = '-';							// ADD 2009.12.24
			}											// ADD 2009.12.24
			if (sd_write(fd, buf, 16) != 16) {			// ADD 2009.12.24
				I2C_LCD_Position(0,0);					// ADD 2009.12.24
				I2C_LCD_PrCString(" <Write error>  ");	// ADD 2009.12.24
			}											// ADD 2009.12.24
														// ADD 2009.12.24
			if ((bef_mi != mi) || (iv >= 60)) {			// ADD 2009.12.24
				bef_mi = mi;							// ADD 2009.12.24
				SD_SET_DATE(yy+2000,mm,dd);				// ADD 2009.12.24
				SD_SET_TIME(hh,mi,ss);					// ADD 2009.12.24
				if (sd_sync(fd) < 0) {					// ADD 2009.12.24
					I2C_LCD_Position(0,0);				// ADD 2009.12.24
					I2C_LCD_PrCString("  <Sync error>  ");	// ADD 2009.12.24
				}										// ADD 2009.12.24
			}											// ADD 2009.12.24
														// ADD 2009.12.24
			I2C_LCD_Icon(9, 0);							// ADD 2009.12.24  Lock icon off
		}												// ADD 2009.12.24

#ifdef DELETE_START						// ADD 2009.12.24
		if (fp != SDCard_1_MAXFILES) {	// file active
			// Buffer entry
			logbuf_yy[logbufnum] = yy;
			logbuf_mm[logbufnum] = mm;
			logbuf_dd[logbufnum] = dd;
			logbuf_hh[logbufnum] = hh;
			logbuf_mi[logbufnum] = mi;
			logbuf_ss[logbufnum] = ss;
			logbuf_t[logbufnum]  = t;
			logbuf_h[logbufnum]  = h;
			logbufnum++;

			// Buffer full, physical write
			if ((logbufnum >= LOGBUFSZ) || (iv >= 60)) {
				int i;

				I2C_LCD_Icon(9, 0x10);	// Lock icon on
				
				for(i = 0; i < logbufnum; i++) {
//					static char buf[40];												// UPD 2009.07.19  static->auto  DEL 2009.08.04
					char buf[20];		// ADD 2009.08.04
					int y2;				// ADD 2009.08.04
					signed char t1;		// ADD 2009.08.04
					signed char t2;		// ADD 2009.08.04
					signed char h1;		// ADD 2009.08.04
					signed char h2;		// ADD 2009.08.04
					
// DELETE NEXT 6 LINES  2009.08.04
//					sprintf(buf, "\"%4d/%02d/%02d %02d:%02d:%02d\",%d.%d,%d.%d\r\n",	// UPD 2009.07.19
//						2000+logbuf_yy[i], logbuf_mm[i], logbuf_dd[i],
//						logbuf_hh[i], logbuf_mi[i], logbuf_ss[i],
//						logbuf_t[i]/10, abs(logbuf_t[i])%10,							// UPD 2009.07.19
//						logbuf_h[i]/10, logbuf_h[i]%10);
//					SDCard_1_fputs(buf, fp);
					y2 = logbuf_yy[i] + 2000;	// ADD 2009.08.04
					t1 = logbuf_t[i] / 10;		// ADD 2009.08.04
					t2 = abs(logbuf_t[i]) % 10;	// ADD 2009.08.04
					h1 = logbuf_h[i] / 10;		// ADD 2009.08.04
					h2 = logbuf_h[i] % 10;		// ADD 2009.08.04
					tsprintf5(buf,				// ADD 2009.08.04
						"\"%4d/%02hhd/%02hhd %02hhd:%02hhd:",	// ADD 2009.08.04
						&y2,					// ADD 2009.08.04
						&logbuf_mm[i],			// ADD 2009.08.04
						&logbuf_dd[i],			// ADD 2009.08.04
						&logbuf_hh[i],			// ADD 2009.08.04
						&logbuf_mi[i]);			// ADD 2009.08.04
					SDCard_1_fputs(buf, fp);	// ADD 2009.08.04
					tsprintf5(buf,				// ADD 2009.08.04
//						"%02hhd\",%hhd.%hhd,%hhd.%hhd\r\n",		// DEL 2009.12.18 ADD 2009.08.04
						"%02hhd\",%3hhd.%hhd,%2hhd.%hhd\r\n",	// ADD 2009.12.18
						&logbuf_ss[i],			// ADD 2009.08.04
						&t1,					// ADD 2009.08.04
						&t2,					// ADD 2009.08.04
						&h1,					// ADD 2009.08.04
						&h2);					// ADD 2009.08.04
					if ((t1 == 0) && (logbuf_t[i] < 0)) {	// ADD 2009.12.18  Special... -0.1 ~ -0.9
						buf[5] = '-';						// ADD 2009.12.18
					}										// ADD 2009.12.18
					SDCard_1_fputs(buf, fp);	// ADD 2009.08.04
				}
				SDCard_1_fflush(fp);
				logbufnum = 0;
				
				I2C_LCD_Icon(9, 0);		// Lock icon off
			}
		}
#endif // DELETE_END					// ADD 2009.12.24


		//
		// Get battery power
		//
		bs = getbatt();					// ADD 2009.07.08   After peak power


		//
		// Wait next log
		//
		for(;;) {
			M8C_ClearWDTAndSleep;

			// Battery icon update/blink
			if (bs < 4) {							// ADD 2009.07.08  Normal level
				if (bbs != bs) {					// ADD 2009.07.08  Status change
					I2C_LCD_Icon(13, bs2icon[bs]);	// ADD 2009.07.08  status lavel -> icon patturn
				}									// ADD 2009.07.08
			} else {								// ADD 2009.07.08  Critical level
				if (ss & 1) {						// ADD 2009.07.08  Blink odd second
					I2C_LCD_Icon(13, 0);			// ADD 2009.07.08
				} else {							// ADD 2009.07.08
					I2C_LCD_Icon(13, bs2icon[bs]);	// ADD 2009.07.08
				}									// ADD 2009.07.08
			}										// ADD 2009.07.08
			bbs = bs;								// ADD 2009.07.08

			// Push switch scan
			if (!(Mode_SW_Data_ADDR & Mode_SW_MASK)) {
				sw++;
				if (sw > 3) {
					dateset();
				}
			} else {
				sw = 0;
			}
			if (!(Incr_SW_Data_ADDR & Incr_SW_MASK)) {
				switch(iv) {
				case 1:
					iv = 10;
					I2C_LCD_Icon(7, 0x10);	// Up ON
					break;
				case 10:
					iv = 60;
					I2C_LCD_Icon(7, 0x08);	// Down ON
					break;
				case 60:
					iv = 600;
					I2C_LCD_Icon(7, 0x18);	// Up/Down ON
					break;
				default:
					iv = 1;
					I2C_LCD_Icon(7, 0x00);	// Up down OFF
					break;
				}
			}
			
			// SDcard detect										// ADD 2009.12.24
			if (!(SD_CD_Data_ADDR & SD_CD_MASK)) {					// ADD 2009.12.24
																	// ADD 2009.12.24
				// Unopen											// ADD 2009.12.24
				if (fd < 0) {										// ADD 2009.12.24
					// File open									// ADD 2009.12.24
					I2C_LCD_Position(1,0);							// ADD 2009.12.24
					I2C_LCD_PrCString("SDcard insert...");			// ADD 2009.12.24
					if (sd_mount(0) == 0) {							// ADD 2009.12.24
						char fname[] = "th.csv";					// ADD 2009.12.24
						char ttl[] = "TimeStamp,Temp[C],Humi[%RH]\r\n";	// ADD 2009.12.24
																	// ADD 2009.12.24
						SD_SET_DATE(yy+2000,mm,dd);					// ADD 2009.12.24
						SD_SET_TIME(hh,mi,ss);						// ADD 2009.12.24
						fd = sd_open(fname, SD_O_APPEND|SD_O_CREAT, 0);	// ADD 2009.12.24
						if (fd >= 0) {								// ADD 2009.12.24
							sd_write(fd, ttl, strlen(ttl));			// ADD 2009.12.24
							I2C_LCD_Icon(6, 0x10);					// ADD 2009.12.24
							I2C_LCD_Position(1,0);					// ADD 2009.12.24
							I2C_LCD_PrCString("Logging start   ");	// ADD 2009.12.24
						} else {									// ADD 2009.12.24
							if (sd_unmount() < 0) {					// ADD 2009.12.24
								I2C_LCD_Position(1,0);				// ADD 2009.12.24
								I2C_LCD_PrCString("Unmount error  *");	// ADD 2009.12.24
							}										// ADD 2009.12.24
						}											// ADD 2009.12.24
					} else {										// ADD 2009.12.24
						I2C_LCD_Position(1,0);						// ADD 2009.12.24
						if (sd_errno == SD_EPERM) {					// ADD 2009.12.24
							I2C_LCD_PrCString("Not response!   ");	// ADD 2009.12.24
						} else if (sd_errno == SD_ENXIO) {			// ADD 2009.12.24
							I2C_LCD_PrCString("Not format!     ");	// ADD 2009.12.24
						} else if (sd_errno == SD_ENODEV) {			// ADD 2009.12.24
							I2C_LCD_PrCString("Unable format!  ");	// ADD 2009.12.24
						} else {									// ADD 2009.12.24
							I2C_LCD_PrCString("Error! code=0x");	// ADD 2009.12.24
							I2C_LCD_PrHexByte(sd_errno);			// ADD 2009.12.24
						}											// ADD 2009.12.24
					}												// ADD 2009.12.24
				}													// ADD 2009.12.24
			// SDcard eject											// ADD 2009.12.24
			} else {												// ADD 2009.12.24
				if (fd >= 0) {										// ADD 2009.12.24
					if (sd_close(fd) < 0) {							// ADD 2009.12.24
						I2C_LCD_Position(1,0);						// ADD 2009.12.24
						I2C_LCD_PrCString("Close error     ");		// ADD 2009.12.24
					}												// ADD 2009.12.24
					if (sd_unmount() < 0) {							// ADD 2009.12.24
						I2C_LCD_Position(1,0);						// ADD 2009.12.24
						I2C_LCD_PrCString("Unmount error   ");		// ADD 2009.12.24
					}												// ADD 2009.12.24
					fd = -1;										// ADD 2009.12.24
					I2C_LCD_Icon(6, 0x00);							// ADD 2009.12.24
				}													// ADD 2009.12.24
			}														// ADD 2009.12.24

#ifdef DELETE_START													// ADD 2009.12.24
			// SDcard detect
			if (!(SD_CD_Data_ADDR & SD_CD_MASK)) {

				if (fp == SDCard_1_MAXFILES) {
					uchar s;
					
					// File open
					I2C_LCD_Position(1,0);
//					printf("SDcard inserted ");						// DEL 2009.08.04
					I2C_LCD_PrCString("SDcard insert...");			// ADD 2009.08.04
					SDCard_1_Select(SDCard_1_ENABLE);
					s = SDCard_1_InitCard();
					if ((s == 0x12) || (s == 0x42)) {
						char fname[8];								// ADD 2009.08.04

//						fp = SDCard_1_fopen("TH.csv","a");			// DEL 2009.08.04
						cstrcpy(fname, "TH.csv");					// ADD 2009.08.04
						fp = SDCard_1_fopen(fname, "a");			// ADD 2009.08.04
						if (fp != SDCard_1_MAXFILES) {
//							SDCard_1_fputs("TimeStamp,Temp[C],Humi[%RH]\r\n", fp);	// DEL 2009.08.04
							SDCard_1_fputcs("TimeStamp,Temp[C],Humi[%RH]\r\n", fp);	// ADD 2009.08.04
							SDCard_1_fflush(fp);
							I2C_LCD_Icon(6, 0x10);		// ArrowBox ON
							I2C_LCD_Position(1,0);
//							printf("Logging start   ");				// DEL 2009.08.04
							I2C_LCD_PrCString("Logging start   ");	// ADD 2009.08.04
						} else {
							I2C_LCD_Position(1,0);
//							printf("SDcard open err ");				// DEL 2009.08.04
							I2C_LCD_PrCString("SDcard open err ");	// ADD 2009.08.04
						}
					} else {
						I2C_LCD_Position(1,0);
//						printf("SDcard FAT32? %02x", s);			// DEL 2009.08.04
						I2C_LCD_PrCString("SDcard FAT32?   ");		// ADD 2009.08.04
					}
				}
	
			// SDcard eject
			} else {
				if (fp != SDCard_1_MAXFILES) {	// file active
					SDCard_1_fclose(fp);
					SDCard_1_Select(SDCard_1_DISABLE);
					fp = SDCard_1_MAXFILES;
					I2C_LCD_Icon(6, 0x00);
				}
			}
#endif // DELETE_END												// ADD 2009.12.24

			// Sleep until 1sec intrrupt
			{
				BYTE ss;				// status save
				BYTE sa;				//
		
				sa = ABF_CR0;			// Analog Output Buffer Control Register 0
				ss = ARF_CR;			// Analog Reference Control Register
				
				ABF_CR0 = 0x00;			// block power off (~-230uA)
				ARF_CR  = 0x00;			// block power off (~-3mA)
				M8C_Sleep;				// clock stop      (~3uA)
				
				ARF_CR  = ss;			// dummy
				ARF_CR  = ss;			// power resume
				ABF_CR0 = sa;			// 
			}

			// Next log timing check
			if  (iv ==   1)										break;
			if ((iv ==  10) && ((ss % 10) == 0))				break;
			if ((iv ==  60) && (ss == 0))						break;
			if ((iv == 600) && (ss == 0) && ((mi % 10) == 0))	break;
		}

	}

// 	Counter8_2_Stop();
//	I2C_LCD_Stop();
//	SDCard_1_Stop();
//	Timer16_1_Stop();
}
